# -*- coding: utf-8 -*-
"""
Created on Tue Sep 13 13:44:52 2016

@author: albert.kurucz
"""
# import sys
import serial
import threading
import logging
import argparse
from time import time
from time import sleep
from binascii import hexlify
import serial.tools.list_ports as lp
try:
    from _version import __version__, __revision__
except ImportError:
    __version__ = "UNKNOWN"
    __revision__ = "UNKNOWN"

default_timeout = 30.0   # 30 seconds

class Timeout(Exception):
    pass

class AddressResponseNotReceived(Exception):
    pass

class InvalidBlockSizeResponse(Exception):
    pass

class SerialPortIsNotFound(Exception):
    pass

class TooManySerialPortsFound(Exception):
    pass

class AvrBootloader(serial.Serial):
    def __init__(self, serial_port):
        logging.info('opening serial port: %s', serial_port)
        self.waiting = True
        serial.Serial.__init__(
            self,
            serial_port,
            115200
            )
        self.timeout = 100
        self.thread = threading.Thread(target=self.start)
        self.thread.start()

    def start(self):
        self.timeout = 100
        while self.waiting:
            chin = self.read(1)
            if chin == '\xf9':
                self.write('\xf6')
                self.flush()
                self.read_signature()

    def read_id(self):
        self.flushInput()
        self.write('I')
        self.flush()
        self.id_read = self.read(3)
        if len(self.id_read) == 3:
            logging.info('version: %d', ord(self.id_read[0]))

    def read_signature(self):
        self.flushInput()
        self.write('s')
        self.flush()
        self.signature_read = self.read(3)
        if len(self.signature_read) == 3:
            logging.info('sig1: %s', hex(ord(self.signature_read[2])))
            logging.info('sig2: %s', hex(ord(self.signature_read[1])))
            logging.info('sig3: %s', hex(ord(self.signature_read[0])))
            if self.signature_read == '\x04\x97\x1e':
                self.waiting = False

    def wait(self, timeout=default_timeout):
        end_time = time() + timeout
        while self.waiting:
            if time() > end_time:
                self.waiting = False
                self.thread.join()
                self.close()
                raise Timeout()
            sleep(0.1)

    def set_address(self, address):
        self.flushInput()
        self.write('A')
        self.write(chr(0xff & (address >> 8)))
        self.write(chr(0xff & address))
        self.flush()
        if self.read(1) != '\r':
            raise AddressResponseNotReceived()
        logging.info('address: 0x%x', address)

    def get_block_size(self):
        self.flushInput()
        self.write('b')
        self.flush()
        self.blocksize_read = self.read(3)
        if len(self.blocksize_read) == 3:
            if self.blocksize_read[0] != 'Y':
                raise InvalidBlockSizeResponse()
            self.blocksize = (
                ord(self.blocksize_read[1]) << 8
                ) | ord(self.blocksize_read[2])
            logging.info('blocksize: %d', self.blocksize)

    def write_block(self, data, memory_type='F'):
        data_len = len(data)
        self.flushInput()
        self.write('B')
        self.write(chr(0xff & (data_len >> 8)))
        self.write(chr(0xff & data_len))
        self.write(memory_type)
        self.write(data)
        self.flush()
        self.checksum_read = self.read(2)
        if len(self.checksum_read) == 2:
            self.checksum = (
                ord(self.checksum_read[0]) << 8
                ) | ord(self.checksum_read[1])
            logging.info('checksum: 0x%x', self.checksum)

    def read_block(self, data_len, memory_type='F'):
        self.flushInput()
        self.write('g')
        self.write(chr(0xff & (data_len >> 8)))
        self.write(chr(0xff & data_len))
        self.write(memory_type)
        self.flush()
        self.data_read = self.read(data_len)
        if len(self.data_read) == data_len:
            logging.info('data_read: %s', hexlify(self.data_read))


def _main():
    logging.basicConfig(level=logging.INFO)
    logging.info('snapengine version: %s', __version__)
    logging.info('snapengine revision: %s', __revision__)
    parser = argparse.ArgumentParser()
    parser.add_argument("--serial_port", help="serial port")
    parser.add_argument("--usb_location", help="usb location")
    parser.add_argument("--usb_pid", type=int, help="usb product id")
    parser.add_argument("--usb_vid", type=int, help="usb vendor id")
    parser.add_argument("--usb_serial_number", help="usb serial number")
    parser.add_argument(
        "--timeout",
        type=float,
        default=default_timeout,
        help="timeout to engage bootloder",
        )
    args = parser.parse_args()
    if args.serial_port:
        logging.info("serial_port: %s", args.serial_port)
    if args.timeout:
        logging.info("timeout: %.3f", args.timeout)
    if args.serial_port is None:
        if args.usb_vid:
            logging.info("usb_vid: %d", args.usb_vid)
        if args.usb_pid:
            logging.info("usb_pid: %d", args.usb_pid)
        if args.usb_serial_number:
            logging.info("usb_serial_number: %s", args.usb_serial_number)
        if args.usb_location:
            logging.info("usb_location: %s", args.usb_location)
        com_ports = lp.comports()
        my_com_ports = []
        for com_port in reversed(com_ports):
            logging.info(
                "%s %d %d %s %s",
                com_port.device,
                com_port.vid,
                com_port.pid,
                repr(com_port.serial_number),
                repr(com_port.location),
                )
            if args.usb_vid:
                if args.usb_vid != com_port.vid:
                    continue
            if args.usb_pid:
                if args.usb_pid != com_port.pid:
                    continue
            if args.usb_serial_number:
                if args.usb_serial_number != com_port.serial_number:
                    continue
            if args.usb_location:
                if args.usb_location != com_port.location:
                    continue
            my_com_ports.append(com_port)
        if len(my_com_ports) == 0:
            raise SerialPortIsNotFound()
        if len(my_com_ports) > 1:
            raise TooManySerialPortsFound()
        args.serial_port = my_com_ports[0].device
    bl = AvrBootloader(args.serial_port)
    bl.wait(args.timeout)
    bl.read_id()
    bl.set_address(0x2f0 >> 1)
    bl.write_block("RMDEMGKBF" + "\xff")
    bl.close()
    logging.info('done')

if __name__ == '__main__':
    _main()
